@@ -1,4 +1,4 @@
-/* $Id: default.c,v 5.2 2001/04/22 12:17:24 bertg Exp $
+/* $Id: default.c,v 5.10 2001/06/02 21:00:22 bertg Exp $
  *
  * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-2001 by
  *
@@ -39,14 +39,13 @@
 # ifdef	__apollo
 #  include <X11/ap_keysym.h>
 # endif
-# ifndef VMS
-#  include <sys/param.h>
-# endif
+# include <sys/param.h>
 #endif
 
 #ifdef _WINDOWS
 # include "NT/winX.h"
 # include "NT/winXXPilot.h"
+# include "NT/winConfig.h"
 #endif
 
 #include "version.h"
@@ -65,36 +64,17 @@
 #include "commonproto.h"
 #include "portability.h"
 #include "talk.h"
+#include "default.h"
+#include "checknames.h"
 
-#ifdef VMS
-# include "strcasecmp.h"
-#endif
-
-#ifdef _WINDOWS
-extern	char	**Argv;
-extern	int	Argc;
-#endif
 
 char default_version[] = VERSION;
 
-extern char *talk_fast_msgs[];	/* talk macros */
-char talk_fast_temp_buf[7];		/* can handle up to 999 fast msgs */
-char *talk_fast_temp_buf_big;
 
-#ifndef	lint
-static char sourceid[] =
-    "@(#)$Id: default.c,v 5.2 2001/04/22 12:17:24 bertg Exp $";
-#endif
 
-#ifdef VMS
-#define DISPLAY_ENV	"DECW$DISPLAY"
-#define DISPLAY_DEF	"::0.0"
-#define KEYBOARD_ENV	"DECW$KEYBOARD"
-#else
 #define DISPLAY_ENV	"DISPLAY"
 #define DISPLAY_DEF	":0.0"
 #define KEYBOARD_ENV	"KEYBOARD"
-#endif
 
 #ifndef PATH_MAX
 #define PATH_MAX	1023
@@ -103,48 +83,33 @@ static char sourceid[] =
 /*
  * Default fonts
  */
-#define GAME_FONT	"-*-times-*-*-*-*-18-*-*-*-*-*-iso8859-1"
-#define MESSAGE_FONT	"-*-times-*-*-*-*-14-*-*-*-*-*-iso8859-1"
-#define SCORE_LIST_FONT	"-*-fixed-bold-*-*-*-15-*-*-*-c-*-iso8859-1"
-#define BUTTON_FONT	"-*-*-bold-o-*-*-14-*-*-*-*-*-iso8859-1"
-#define TEXT_FONT	"-*-*-bold-i-*-*-14-*-*-*-p-*-iso8859-1"
-#define TALK_FONT	"-*-fixed-bold-*-*-*-15-*-*-*-c-*-iso8859-1"
-#define KEY_LIST_FONT	"-*-fixed-medium-r-*-*-10-*-*-*-c-*-iso8859-1"
-
-#ifndef _WINDOWS
+#define GAME_FONT	"-*-times-*-*-*--18-*-*-*-*-*-iso8859-1"
+#define MESSAGE_FONT	"-*-times-*-*-*--14-*-*-*-*-*-iso8859-1"
+#define SCORE_LIST_FONT	"-*-fixed-bold-*-*--15-*-*-*-c-*-iso8859-1"
+#define BUTTON_FONT	"-*-*-bold-o-*--14-*-*-*-*-*-iso8859-1"
+#define TEXT_FONT	"-*-*-bold-i-*--14-*-*-*-p-*-iso8859-1"
+#define TALK_FONT	"-*-fixed-bold-*-*--15-*-*-*-c-*-iso8859-1"
+#define KEY_LIST_FONT	"-*-fixed-medium-r-*--10-*-*-*-c-*-iso8859-1"
 #define MOTD_FONT	"-*-courier-bold-r-*--14-*-*-*-*-*-iso8859-1"
-#else
-#define MOTD_FONT	"-*-courier-bold-r-*-*-14-*-*-*-*-*-iso8859-1"
-#endif
+
 
 char			myName[] = "xpilot";
 char			myClass[] = "XPilot";
 
+
 #ifdef SPARC_CMAP_HACK
 char  frameBuffer[MAX_CHARS]; /* frame buffer */
 #endif
 
-#ifdef _WINDOWS
-	const char*	winHelpFile;
-	#ifdef	_DEBUG
-	#define	WINDOWSDEBUGPATH(__x)						\
-		{												\
-			char* p = texturePath;						\
-			texturePath = malloc(strlen(p) + 13);		\
-			strcpy(texturePath, "..\\..\\..\\..\\");	\
-			strcat(texturePath, p);						\
-			free(p);									\
-		}
-	#else
-	#define	WINDOWSDEBUGPATH(__x)
-	#endif
-#else
-#define	WINDOWSDEBUGPATH(__x)
-#endif
 
-#if DEVELOPMENT
-static void Start_testing(char *testBuffer);
-#endif
+
+extern char *talk_fast_msgs[];	/* talk macros */
+char talk_fast_temp_buf[7];		/* can handle up to 999 fast msgs */
+char *talk_fast_temp_buf_big;
+
+
+static void Get_test_resources(XrmDatabase rDB);
+
 
 keys_t buttonDefs[MAX_POINTER_BUTTONS];
 
@@ -161,14 +126,7 @@ extern char conf_soundfile_string[];
  * Help lines can span multiple lines, but for
  * the key help window only the first line is used.
  */
-struct option {
-    const char		*name;		/* option name */
-    const char		*noArg;		/* value for non-argument options */
-    const char		*fallback;	/* default value */
-    keys_t		key;		/* key if not KEY_DUMMY */
-    const char		*help;		/* user help (multiline) */
-    unsigned		hash;		/* option name hashed. */
-} options[] = {
+option options[] = {
     {
 	"help",
 	"Yes",
@@ -190,6 +148,20 @@ struct option {
 	KEY_DUMMY,
 	"Set the nickname.\n"
     },
+    {
+	"user",
+	NULL,
+	"",
+	KEY_DUMMY,
+	"Set the realname.\n"
+    },
+    {
+	"host",
+	NULL,
+	"",
+	KEY_DUMMY,
+	"Set the hostname.\n"
+    },
     {
 	"join",
 	"Yes",
@@ -2077,8 +2049,9 @@ struct option {
     },
 };
 
+int optionsCount = NELEM(options);
 
-static unsigned String_hash(const char *s)
+unsigned String_hash(const char *s)
 {
     unsigned		hash = 0;
 
@@ -2097,8 +2070,7 @@ char* Get_keyHelpString(keys_t key)
 
     for (i = 0; i < NELEM(options); i++) {
 	if (options[i].key == key) {
-	    strncpy(buf, options[i].help, sizeof buf);
-	    buf[sizeof buf - 1] = '\0';
+	    strlcpy(buf, options[i].help, sizeof buf);
 	    if ((nl = strchr(buf, '\n')) != NULL) {
 		*nl = '\0';
 	    }
@@ -2173,28 +2145,12 @@ static void Usage(void)
     exit(1);
 }
 
-#ifdef _WINDOWS
-void	GetWindowsProfileString(const char* section, const char* key,
-								const char* def, char* result, int size)
-{
-	int		i;
-
-	for (i=0; i<3; i++)
-	{
-		GetPrivateProfileString("Settings", key, "", result, size, 
-			Get_xpilotini_file(i));
-		if (result[0] != '\0')
-			return;
-	}
-	strncpy(result, def, size);
-}
-#endif
 
 static int Find_resource(XrmDatabase db, const char *resource,
 			 char *result, unsigned size, int *index)
 {
-    int			i;
 #ifndef _WINDOWS
+    int			i;
     int			len;
     char		str_name[80],
 			str_class[80],
@@ -2222,44 +2178,17 @@ static int Find_resource(XrmDatabase db, const char *resource,
 	    len = 0;
 	} else {
 	    len = MIN(rmValue.size, size - 1);
-	    strncpy(result, rmValue.addr, len);
+	    memcpy(result, rmValue.addr, len);
 	}
 	result[len] = '\0';
 	return 1;
     }
-    strncpy(result, options[*index].fallback, size);
-    result[size - 1] = '\0';
+    strlcpy(result, options[*index].fallback, size);
 
     return 0;
 
 #else	/* _WINDOWS */
-
-    unsigned		hash = String_hash(resource);
-
-    for (i = 0;;) {
-	if (!strcmp(resource, options[i].name)) {
-	    *index = i;
-	    break;
-	}
-	if (++i >= NELEM(options)) {
-	    errno = 0;
-	    error("BUG: Can't find option \"%s\"", resource);
-	    exit(1);
-	}
-    }
-    GetWindowsProfileString("Settings", resource, options[*index].fallback, result, size);
-
-#if 0
-    GetPrivateProfileString("Settings", resource, "", result, size, 
-			    Get_xpilotini_file(1));
-    if (result[0] == '\0') {
-	GetPrivateProfileString("Settings", resource, "", result, size, 
-				Get_xpilotini_file(2));
-	if (result[0] == '\0') {
-	    strncpy(result, options[*index].fallback, size);
-	}
-    }
-#endif
+    Config_get_resource(resource, result, size, index);
 
     return 1;
 #endif
@@ -2315,23 +2244,18 @@ static void Get_float_resource(XrmDatabase db,
 			       const char *resource, DFLOAT *result)
 {
     int			index;
+    double		temp_result;
     char		resValue[MAX_CHARS];
 
+    temp_result = 0.0;
     Find_resource(db, resource, resValue, sizeof resValue, &index);
-#ifndef _WINDOWS
-    if (sscanf(resValue, "%f", result) <= 0) {
-#else
-    if (sscanf(resValue, "%lf", result) <= 0) {
-#endif
+    if (sscanf(resValue, "%lf", &temp_result) <= 0) {
 	errno = 0;
 	error("Bad value \"%s\" for option \"%s\", using default...",
 	      resValue, resource);
-#ifndef _WINDOWS
-	sscanf(options[index].fallback, "%f", result);
-#else
-	sscanf(options[index].fallback, "%lf", result);
-#endif
+	sscanf(options[index].fallback, "%lf", &temp_result);
     }
+    *result = (DFLOAT) temp_result;
 }
 
 
@@ -2357,6 +2281,49 @@ static void Get_bit_resource(XrmDatabase db, const char *resource,
     }
 }
 
+static void Get_shipshape_resource(XrmDatabase db, char **ship_shape)
+{
+    char		resValue[MAX(2*MSG_LEN, PATH_MAX + 1)];
+
+    Get_resource(db, "shipShape", resValue, sizeof resValue);
+    *ship_shape = xp_strdup(resValue);
+    if (*ship_shape && **ship_shape && !strchr(*ship_shape, '(' )) {
+	/* so it must be the name of shipshape defined in the shipshapefile. */
+	Get_resource(db, "shipShapeFile", resValue, sizeof resValue);
+	if (resValue[0] != '\0') {
+	    FILE *fp = fopen(resValue, "r");
+	    if (!fp) {
+		perror(resValue);
+	    } else {
+		char *ptr;
+		char *str;
+		char line[1024];
+		while (fgets(line, sizeof line, fp)) {
+		    if ((str = strstr(line, "(name:" )) != NULL
+			|| (str = strstr(line, "(NM:" )) != NULL) {
+			str = strchr(str, ':');
+			while (*++str == ' ');
+			if ((ptr = strchr(str, ')' )) != NULL) {
+			    *ptr = '\0';
+			}
+			if (!strcmp(str, *ship_shape)) {
+			    /* Gotcha */
+			    free(*ship_shape);
+			    if (ptr != NULL) {
+				*ptr = ')';
+			    }
+			    *ship_shape = xp_strdup(line);
+			    break;
+			}
+		    }
+		}
+		fclose(fp);
+	    }
+	}
+    }
+}
+
+
 #ifndef _WINDOWS
 void Get_xpilotrc_file(char *path, unsigned size)
 {
@@ -2365,89 +2332,21 @@ void Get_xpilotrc_file(char *path, unsigned size)
     const char		*optionalFile = getenv("XPILOTRC");
 
     if (optionalFile != NULL) {
-	strncpy(path, optionalFile, size);
+	strlcpy(path, optionalFile, size);
     }
     else if (home != NULL) {
-	strncpy(path, home, size);
-	strncat(path, "/", size);
-	strncat(path, defaultFile, size);
+	strlcpy(path, home, size);
+	strlcat(path, "/", size);
+	strlcat(path, defaultFile, size);
     }
     else {
-	strncpy(path, "", size);
+	strlcpy(path, "", size);
     }
-    if (size > 0) {
-	path[size - 1] = '\0';
-    }
-}
-#else
-/* Get the name for XPilot.ini .  Hopefully, this will be the fully qualified
-path to where XPilot.exe started from.  Right now, we guess that is the location
-of the default help file.
-the parameter level is used to determine which ini file to use.  This allows us
-to cascade through a series of default ini files.
-level = -1 = free private memory.
-level =  0 = current directory, XPilot.host.ini  (currently returns defini)
-level =  1 = current directory, XPilot.ini
-level =  2 = Windows directory, XPilot.ini
-*/
-char* Get_xpilotini_file(int level)
-{
-	static	char*	xpini = NULL;
-	static	char*	winxpini = NULL;
-	static	char*	defini = "XPilot.ini";
-
-	char*	end;
-	int		size;
-
-	switch(level)
-	{
-	case 0:
-	case 1:
-		if (xpini)
-			return(xpini);
-		if (!winHelpFile)			/* do we have the help default to build from? */
-			return(defini);			/* no, return generic name (wherever that is) */
-
-		end = strrchr(winHelpFile, '\\');
-		if (!end)
-			return(defini);			/* no backslash? bail out */
-		size = (end-winHelpFile) + 13;
-		xpini = malloc(size+1);
-		memset(xpini, 0, size);
-		strncpy(xpini, winHelpFile, end-winHelpFile);
-		strcat(xpini, "\\");
-		strcat(xpini, defini);
-		return(xpini);
-	case 2:
-		if (winxpini)
-			return(winxpini);
-		size = GetWindowsDirectory(winxpini, 0);	/* returns size needed */
-		winxpini = malloc(size+15);
-		GetWindowsDirectory(winxpini, size+13);
-		if (winxpini[strlen(winxpini)] != '\\')
-			strcat(winxpini, "\\");
-		strcat(winxpini, defini);
-		return(winxpini);
-	case -1:
-		if (xpini)
-		{
-			free(xpini);
-			xpini = NULL;
-		}
-		if (winxpini)
-		{
-			free(winxpini);
-			winxpini = NULL;
-		}
-		return(NULL);
-	}
-	error("BUG: bad level in Get_xpilotini_file");
-	return(defini);
 }
 #endif
 
-#ifndef _WINDOWS
 
+#ifndef _WINDOWS
 static void Get_file_defaults(XrmDatabase *rDBptr)
 {
     int			len;
@@ -2460,15 +2359,6 @@ static void Get_file_defaults(XrmDatabase *rDBptr)
     sprintf(path, "%s%s", Conf_libdir(), myClass);
     *rDBptr = XrmGetFileDatabase(path);
 
-#ifdef VMS
-    /*
-     * None of the paths generated will be valid VMS file names.
-     */
-    tmpDB = XrmGetFileDatabase("SYS$LOGIN:decw$xdefaults.dat");
-    XrmMergeDatabases(tmpDB, rDBptr);
-    tmpDB = XrmGetFileDatabase("DECW$USER_DEFAULTS:xpilot.dat");
-    XrmMergeDatabases(tmpDB, rDBptr);
-#else
     if (lang != NULL) {
 	sprintf(path, "/usr/lib/X11/%s/app-defaults/%s", lang, myClass);
 	if (access(path, 0) == -1) {
@@ -2538,7 +2428,6 @@ static void Get_file_defaults(XrmDatabase *rDBptr)
 	tmpDB = XrmGetFileDatabase(path);
 	XrmMergeDatabases(tmpDB, rDBptr);
     }
-#endif
 }
 #endif	/* _WINDOWS*/
 
@@ -2546,7 +2435,8 @@ static void Get_file_defaults(XrmDatabase *rDBptr)
 void Parse_options(int *argcp, char **argvp, char *realName, int *port,
 		   int *my_team, int *text, int *list,
 		   int *join, int *noLocalMotd,
-		   char *nickName, char *dispName, char *shut_msg)
+		   char *nickName, char *dispName, char *hostName,
+		   char *shut_msg)
 {
     char		*ptr;
     char		*str;
@@ -2557,14 +2447,9 @@ void Parse_options(int *argcp, char **argvp, char *realName, int *port,
     keys_t		key;
     KeySym		ks;
 
-#ifdef VMS
-    char		resValue[PATH_MAX + 1];
-#else
     char		resValue[MAX(2*MSG_LEN, PATH_MAX + 1)];
-#endif
     XrmDatabase		argDB = 0, rDB = 0;
 
-    extern void		Record_init(char *filename);
 #ifndef _WINDOWS
     XrmOptionDescRec	*xopt;
     int			size;
@@ -2635,17 +2520,16 @@ void Parse_options(int *argcp, char **argvp, char *realName, int *port,
     if (Get_string_resource(argDB, "display", dispName, MAX_DISP_LEN) == 0
 	|| dispName[0] == '\0') {
 	if ((ptr = getenv(DISPLAY_ENV)) != NULL) {
-	    strncpy(dispName, ptr, MAX_DISP_LEN);
-	    dispName[MAX_DISP_LEN - 1] = '\0';
+	    strlcpy(dispName, ptr, MAX_DISP_LEN);
 	} else {
-	    strcpy(dispName, DISPLAY_DEF);
+	    strlcpy(dispName, DISPLAY_DEF, MAX_DISP_LEN);
 	}
     }
     if ((dpy = XOpenDisplay(dispName)) == NULL) {
 	error("Can't open display '%s'", dispName);
 	if (strcmp(dispName, "NO_X") == 0) {
 	    /* user does not want X stuff.  experimental.  use at own risk. */
-	    strcpy(nickName, realName);
+	    strlcpy(nickName, realName, MAX_NAME_LEN);
 	    *my_team = TEAM_NOT_SET;
 	    Get_int_resource(argDB, "port", port);
 	    Get_bool_resource(argDB, "list", list);
@@ -2662,8 +2546,7 @@ void Parse_options(int *argcp, char **argvp, char *realName, int *port,
     if (Get_string_resource(argDB, "keyboard", resValue, MAX_DISP_LEN) == 0
 	|| resValue[0] == '\0') {
 	if ((ptr = getenv(KEYBOARD_ENV)) != NULL) {
-	    strncpy(resValue, ptr, MAX_DISP_LEN);
-	    resValue[MAX_DISP_LEN - 1] = '\0';
+	    strlcpy(resValue, ptr, MAX_DISP_LEN);
 	}
     }
     if (resValue[0] == '\0') {
@@ -2685,9 +2568,7 @@ void Parse_options(int *argcp, char **argvp, char *realName, int *port,
 
     Get_string_resource(rDB, "geometry", resValue, sizeof resValue);
     geometry = xp_strdup(resValue);
-#else	/* _WINDOWS */
-    /* Windows needs to know about -serverIni first */
-#endif	/* _WINDOWS */
+#endif
 
     if ((talk_fast_temp_buf_big = (char *)malloc(TALK_FAST_MSG_SIZE)) != NULL) {
         for (i = 0; i < TALK_FAST_NR_OF_MSGS; ++i) {
@@ -2705,9 +2586,21 @@ void Parse_options(int *argcp, char **argvp, char *realName, int *port,
 
     Get_bool_resource(rDB, "ignoreWindowManager", &ignoreWindowManager);
 
+    Get_resource(rDB, "user", resValue, MAX_NAME_LEN);
+    if (resValue[0]) {
+	strlcpy(realName, resValue, MAX_NAME_LEN);
+    }
+    Fix_real_name(realName);
+
+    Get_resource(rDB, "host", resValue, MAX_HOST_LEN);
+    if (resValue[0]) {
+	strlcpy(hostName, resValue, MAX_HOST_LEN);
+    }
+    Fix_host_name(hostName);
+
     Get_resource(rDB, "name", nickName, MAX_NAME_LEN);
     if (!nickName[0]) {
-	strcpy(nickName, realName);
+	strlcpy(nickName, realName, MAX_NAME_LEN);
     }
     CAP_LETTER(nickName[0]);
     if (nickName[0] < 'A' || nickName[0] > 'Z') {
@@ -2724,27 +2617,14 @@ void Parse_options(int *argcp, char **argvp, char *realName, int *port,
 	    break;
 	}
     }
-    strncpy(realname, realName, sizeof(realname) - 1);
-    strncpy(name, nickName, sizeof(name) - 1);
+    strlcpy(realname, realName, sizeof(realname));
+    strlcpy(name, nickName, sizeof(name));
 
-#ifdef _WINDOWS
-    if (*name == '\0') {
-	/* Windows can have no default name */
-	strcpy(name, "NoName");
-    }
-    if (Argc > 2) {
-	/* oh, it's those fixed pos variables again! */
-	if (Argv[2] && !strncmp(Argv[2], "-team", 5)) {
-	    /* i've really got to fix that... */
-	    *my_team = Argv[2][6] & 0x0f;
-	}
-	else {
-	    Get_int_resource(rDB, "team", my_team);
-	}
-    }
-#else
     Get_int_resource(rDB, "team", my_team);
-#endif
+
+    IFWINDOWS( Config_get_name(name); )
+    IFWINDOWS( Config_get_team(my_team); )
+
     if (*my_team < 0 || *my_team > 9) {
 	*my_team = TEAM_NOT_SET;
     }
@@ -2760,42 +2640,7 @@ void Parse_options(int *argcp, char **argvp, char *realName, int *port,
     Get_bool_resource(rDB, "refreshMotd", &i);
     refreshMotd = (i != 0) ? true : false;
 
-    Get_resource(rDB, "shipShape", resValue, sizeof resValue);
-    shipShape = xp_strdup(resValue);
-    if (shipShape && *shipShape && !strchr(shipShape, '(' )) {
-	/* so it must be the name of shipshape defined in the shipshapefile. */
-	Get_resource(rDB, "shipShapeFile", resValue, sizeof resValue);
-	if (resValue[0] != '\0') {
-	    FILE *fp = fopen(resValue, "r");
-	    if (!fp) {
-		perror(resValue);
-	    } else {
-		char line[1024];
-		while (fgets(line, sizeof line, fp)) {
-		    if ((str = strstr(line, "(name:" )) != NULL
-			|| (str = strstr(line, "(NM:" )) != NULL) {
-			str = strchr(str, ':');
-			while (*++str == ' ');
-			if ((ptr = strchr(str, ')' )) != NULL) {
-			    *ptr = '\0';
-			}
-			if (!strcmp(str, shipShape)) {
-			    /* Gotcha */
-			    free(shipShape);
-			    if (ptr != NULL) {
-				*ptr = ')';
-			    }
-			    shipShape = xp_strdup(line);
-			    break;
-			}
-		    }
-		}
-#ifndef _WINDOWS
-		fclose(fp);
-#endif
-	    }
-	}
-    }
+    Get_shipshape_resource(rDB, &shipShape);
     Validate_shape_str(shipShape);
 
     Get_float_resource(rDB, "power", &power);
@@ -2846,10 +2691,10 @@ void Parse_options(int *argcp, char **argvp, char *realName, int *port,
     colorSwitch = (i != 0) ? true : false;
     Get_bool_resource(rDB, "multibuffer", &i);
     multibuffer = (i != 0) ? true : false;
-#ifndef _WINDOWS
-	/* Windows already derived maxColors in InitWinX */
-    Get_int_resource(rDB, "maxColors", &maxColors);
-#endif
+
+    /* Windows already derived maxColors in InitWinX */
+    IFNWINDOWS( Get_int_resource(rDB, "maxColors", &maxColors); )
+
     Get_string_resource(rDB, "black", color_names[0], sizeof(color_names[0]));
     Get_string_resource(rDB, "white", color_names[1], sizeof(color_names[1]));
     Get_string_resource(rDB, "blue", color_names[2], sizeof(color_names[2]));
@@ -2859,10 +2704,10 @@ void Parse_options(int *argcp, char **argvp, char *realName, int *port,
 	sprintf(buf, "color%d", i);
 	if (!Get_string_resource(rDB, buf, resValue, MAX_COLOR_LEN)) {
 	    if (i < NUM_COLORS) {
-		strcpy(resValue, color_names[i]);
+		strlcpy(resValue, color_names[i], MAX_COLOR_LEN);
 	    }
 	}
-	strcpy(color_names[i], resValue);
+	strlcpy(color_names[i], resValue, MAX_COLOR_LEN);
     }
     Get_int_resource(rDB, "hudColor", &hudColor);
     Get_int_resource(rDB, "hudLockColor", &hudLockColor);
@@ -2936,7 +2781,6 @@ void Parse_options(int *argcp, char **argvp, char *realName, int *port,
     Record_init(resValue);
     Get_resource(rDB, "texturePath", resValue, sizeof resValue);
     texturePath = xp_strdup(resValue);
-	WINDOWSDEBUGPATH(texturePath);
     Get_resource(rDB, "wallTextureFile", resValue, sizeof resValue);
     wallTextureFile = xp_strdup(resValue);
     Get_resource(rDB, "decorTextureFile", resValue, sizeof resValue);
@@ -2947,15 +2791,9 @@ void Parse_options(int *argcp, char **argvp, char *realName, int *port,
     Get_int_resource(rDB, "maxFPS", &maxFPS);
     oldMaxFPS = maxFPS;
 
-#ifdef _WINDOWS
-    /*
-     * Windows specific things
-     */
-    {
-	Get_int_resource(rDB, "radarDivisor", &RadarDivisor);
-	Get_bool_resource(rDB, "threadedDraw", &ThreadedDraw);
-    }
-#endif	
+    IFWINDOWS( Get_int_resource(rDB, "radarDivisor", &RadarDivisor); )
+    IFWINDOWS( Get_bool_resource(rDB, "threadedDraw", &ThreadedDraw); )
+
 #ifdef	WINDOWSCALING
     Get_float_resource(rDB, "scaleFactor", &scaleFactor);
     if (scaleFactor == 0.0) {
@@ -2968,6 +2806,7 @@ void Parse_options(int *argcp, char **argvp, char *realName, int *port,
     }
     LIMIT(scaleFactor_s, MIN_SCALEFACTOR, MAX_SCALEFACTOR);
 #endif
+
 #ifdef SOUND
     Get_string_resource(rDB, "sounds", sounds, sizeof sounds);
     Get_int_resource(rDB, "maxVolume", &maxVolume);
@@ -2978,14 +2817,7 @@ void Parse_options(int *argcp, char **argvp, char *realName, int *port,
     Get_string_resource(rDB, "frameBuffer", frameBuffer, sizeof frameBuffer);
 #endif
 
-#ifdef DEVELOPMENT
-    {
-	char testBuffer[256];
-	Get_string_resource(rDB, "test", testBuffer, sizeof testBuffer);
-	Start_testing(testBuffer);
-    }
-#endif
-
+    Get_test_resources(rDB);
 
     /*
      * Key bindings
@@ -3093,9 +2925,7 @@ void Parse_options(int *argcp, char **argvp, char *realName, int *port,
 
 void	defaultCleanup(void)
 {
-#ifdef _WINDOWS
-    Get_xpilotini_file(-1);
-#endif
+    IFWINDOWS( Get_xpilotini_file(-1); )
 
     if (keyDefs) {
 	free(keyDefs);
@@ -3153,9 +2983,12 @@ static void X_after(Display *display)
     }
 }
 
-static void Start_testing(char *testBuffer)
+static void Get_test_resources(XrmDatabase rDB)
 {
     char	*s;
+    char testBuffer[256];
+
+    Get_string_resource(rDB, "test", testBuffer, sizeof testBuffer);
 
     for (s = strtok(testBuffer, ":"); s != NULL; s = strtok(NULL, ":")) {
 	if (!strncasecmp(s, "xsync", 3)) {
@@ -3166,10 +2999,13 @@ static void Start_testing(char *testBuffer)
 	    XSetErrorHandler(X_error_handler);
 	}
 	else if (!strncasecmp(s, "after", 5)) {
-	    XSetAfterFunction(dpy, (int (*)()) X_after);
+	    XSetAfterFunction(dpy, (int (*)(
+#if NeedNestedPrototypes
+					    Display *
+#endif
+					    )) X_after);
 	}
 	else if (!strncasecmp(s, "color", 3)) {
-	    extern void Colors_debug(void);
 	    Colors_debug();
 	}
 	else {
@@ -3178,5 +3014,9 @@ static void Start_testing(char *testBuffer)
 	}
     }
 }
+#else
+static void Get_test_resources(XrmDatabase rDB)
+{
+}
 #endif
 
